#!/usr/bin/env python3
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os

import amulet

from charmhelpers.contrib.openstack.amulet.utils import (
    OpenStackAmuletUtils,
    DEBUG,  # flake8: noqa
    ERROR
)

# Use DEBUG to turn on debug logging
u = OpenStackAmuletUtils(DEBUG)

SETUP_TIMEOUT = 9600


class AgentMinimumDeployment():
    def __init__(self, ubuntu_series, openstack_origin=None,
                 midonet_origin=None):
        self.os_release = openstack_origin
        self.series = ubuntu_series
        self.midonet_origin = midonet_origin
        self.mem = self.midonet_origin[:3] == 'mem'
        self.mem_credentials = {'mem-username': os.getenv('MEM_USERNAME'),
                                'mem-password': os.getenv('MEM_PWD')}
        self.d = amulet.Deployment(series=self.series)
        self.d.charm_name = 'neutron-agents-midonet'

        # Sentries
        self.zookeeper_sentry = None
        self.keystone_sentry = None
        self.cassandra_sentry = None
        self.nova_compute_sentry = None
        self.mn_api_sentry = None
        self.agents_mn_sentry = None
        self.keystone_api_relation = None
        self.zookeeper_api_relation = None

        self._add_services()
        self._add_relations()
        self._configure_services()
        self._deploy()
        self._initialize_tests()

    def _add_services(self):
        self.d.add('mysql', charm='mysql', units=1,
                   series=self.series)
        self.d.add('keystone', charm='keystone', units=1, series=self.series)
        self.d.add('cassandra', charm='cassandra',
                   units=1, series=self.series)
        self.d.add('zookeeper', charm='zookeeper', units=1,
                   series=self.series)
        self.d.add('rabbitmq', charm='rabbitmq-server', units=1,
                   series=self.series)
        self.d.add('nova', charm='nova-cloud-controller', units=1,
                   series=self.series)
        self.d.add('neutron',
                   charm='cs:~openstack-charmers-next/trusty/neutron-api',
                   units=1,
                   series=self.series)
        self.d.add('midonet-api', charm='midonet-api',
                   branch='lp:~celebdor/charms/trusty/midonet-api/trunk',
                   series=self.series)
        self.d.add('midonet-agent', charm='midonet-agent',
                   branch='lp:~celebdor/charms/trusty/midonet-agent/trunk',
                   series=self.series)
        self.d.add('neutron-agents-midonet', constraints={'mem': "4G"})

    def _add_relations(self):
        # Keystone
        self.d.relate('mysql:shared-db', 'keystone:shared-db')

        # Nova
        self.d.relate('mysql:shared-db', 'nova:shared-db')
        self.d.relate('keystone:identity-service', 'nova:identity-service')
        self.d.relate('rabbitmq:amqp', 'nova:amqp')
        self.d.relate('neutron-agents-midonet:neutron_agents',
                      'nova:quantum-network-service')
        self.d.relate('neutron:neutron-api', 'nova:neutron-api')

        # Neutron
        self.d.relate('mysql:shared-db', 'neutron:shared-db')
        self.d.relate('keystone:identity-service', 'neutron:identity-service')
        self.d.relate('rabbitmq:amqp', 'neutron:amqp')
        self.d.relate('neutron-agents-midonet:neutron-plugin-api',
                      'neutron:neutron-plugin-api')
        self.d.relate('midonet-api:midonet-api', 'neutron:midonet')

        # MidoNet
        self.d.relate('keystone:identity-service', 'midonet-api:keystone')
        self.d.relate('zookeeper:zookeeper', 'midonet-api:zookeeper')
        self.d.relate('midonet-agent:host', 'midonet-api:host')
        self.d.relate('rabbitmq:amqp', 'neutron-agents-midonet:amqp')
        self.d.relate('midonet-agent:host', 'neutron-agents-midonet:agent')
        self.d.relate('midonet-api:midonet-api',
                      'neutron-agents-midonet:midonet')
        self.d.relate('cassandra:database', 'midonet-agent:cassandra')
        self.d.relate('zookeeper:zookeeper', 'midonet-agent:zookeeper')

    def _configure_services(self):
        self.d.configure('keystone', {
            'admin-user': 'admin',
            'admin-password': 'openstack',
            'admin-token': 'osttesting',
            'enable-pki': 'false',
            'openstack-origin': self.os_release})

        self.d.configure('nova', {
            'network-manager': 'Neutron',
            'openstack-origin': self.os_release})

        neutron_conf = {
            'l2-population': 'false',
            'neutron-security-groups': 'true',
            'neutron-plugin': 'midonet',
            'neutron-external-network': 'Public_Network',
            'midonet-origin': self.midonet_origin,
            'openstack-origin': self.os_release}
        if self.mem:
            neutron_conf.update(self.mem_credentials)

        self.d.configure('neutron', neutron_conf)

        self.d.configure('cassandra', {
            'authenticator': 'AllowAllAuthenticator',
            'cluster_name': 'midonet',
            'install_keys': '7E41C00F85BFC1706C4FFFB3350200F2B999A372',
            'install_sources':
            'deb http://debian.datastax.com/community 2.0 main',
            'extra_packages': 'dsc20'})

        self.d.configure('mysql', {'max-connections': 500})

        api_config = {'midonet-origin': self.midonet_origin,
                      'openstack-origin': self.os_release}
        if self.mem:
            api_config.update(self.mem_credentials)
        self.d.configure('midonet-api', api_config)
        self.d.configure('midonet-agent', api_config)
        self.d.configure('neutron-agents-midonet', api_config)


    def _deploy(self):
        try:
            self.d.setup(timeout=SETUP_TIMEOUT)
            self.d.sentry.wait(timeout=SETUP_TIMEOUT)
        except amulet.helpers.TimeoutError:
            amulet.raise_status(amulet.SKIP,
                                msg="Environment wasn't stood up in time")

    def _initialize_tests(self):
        self.zookeeper_sentry = self.d.sentry.unit['zookeeper/0']
        self.keystone_sentry = self.d.sentry.unit['keystone/0']
        self.mn_api_sentry = self.d.sentry.unit['midonet-api/0']
        self.mn_agent_sentry = self.d.sentry.unit['midonet-agent/0']
        self.agents_mn_sentry = self.d.sentry.unit['neutron-agents-midonet/0']

        self.keystone_api_relation = self.keystone_sentry.relation(
            'identity-service', 'midonet-api:keystone')
        self.zookeeper_api_relation = self.zookeeper_sentry.relation(
            'zookeeper', 'midonet-api:zookeeper')

    def run_tests(self):
        for test in dir(self):
            if test.startswith('test_'):
                getattr(self, test)()

    def test_neutron_agents_mn(self):
        u.log.debug('Checking if the key has been correctly fetched...')
        apt_key_out, _ = self.mn_agent_sentry.run('apt-key list | '
                                                  'grep Midokura')
        if 'Midokura' not in apt_key_out:
            amulet.raise_status(amulet.FAIL,
                                msg='Midokura repository key is not installed')

        u.log.debug('Checking if midonet-plugin package has been installed...')
        dpkg_plugin_out, _ = self.agents_mn_sentry.run(
            'dpkg -l | grep python-neutron-plugin-midonet')
        if 'python-neutron-plugin-midonet' not in dpkg_plugin_out:
            amulet.raise_status(amulet.FAIL,
                                msg='python-neutron-plugin-midonet package is '
                                'not installed')

        u.log.debug('Checking neutron-dhcp-agent...')
        dhcp_agent_out, _ = self.agents_mn_sentry.run(
            'service neutron-dhcp-agent status')
        if 'running' not in dhcp_agent_out:
            amulet.raise_status(amulet.FAIL,
                                msg="neutron-dhcp-agent not running: %s" %
                                dhcp_agent_out)

        u.log.debug('Checking neutron-metadata-agent...')
        meta_agent_out, _ = self.agents_mn_sentry.run(
            'service neutron-metadata-agent status')
        if 'running' not in meta_agent_out:
            amulet.raise_status(amulet.FAIL,
                                msg="neutron-metadata-agent not running: %s" %
                                meta_agent_out)
        u.log.debug('Checking nova-api-metadata...')
        meta_agent_out, _ = self.agents_mn_sentry.run(
            'service nova-api-metadata status')
        if 'running' not in meta_agent_out:
            amulet.raise_status(amulet.FAIL,
                                msg="nova-api-metadata not running: %s" %
                                meta_agent_out)
