#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import shutil
import tempfile

import services

from .midonet_helpers import test

HOOKENV_PATCHES = [
    'charm_dir',
    'config',
    'is_relation_made',
    'log',
    'open_port',
    'relation_get',
    'relation_ids',
    'related_units',
    'relation_set',
    'unit_get']


class HookTests(test.CharmTestCase):
    def __init__(self, *args, **kwargs):
        self.nova = ['ks_host',
                     1111,
                     'http',
                     'http',
                     2222,
                     'me',
                     'mypass',
                     'tennant',
                     'https://ks_host:2222/v2.0',
                     'ks_host',
                     'tennant',
                     'RegionOne',
                     'http://quantum_host:3333',
                     'quantum_host',
                     3333,
                     '10.10.10.11']
        test.CharmTestCase.__init__(self, *args, **kwargs)

    def setUp(self):
        self.relations = test.FakeRelation(self._full_relation_data())
        test.CharmTestCase.setUp(self, services.hookenv, HOOKENV_PATCHES)

        def lsb_release():
            return {
                'DISTRIB_ID': 'Ubuntu',
                'DISTRIB_RELEASE': '14.04',
                'DISTRIB_CODENAME': 'trusty',
                'DISTRIB_DESCRIPTION': "Ubuntu 14.04.3 LTS",
            }

        def get_os_codename_package(pkg, fatal=True):
            return 'liberty'

        self.patch_all(services.utils,
                       ('get_os_codename_package',
                        'lsb_release',))
        self.lsb_release.side_effect = lsb_release
        self.get_os_codename_package.side_effect = get_os_codename_package

        self.patch_all(services.base, ('service_restart', 'service_stop'))
        self.patch_all(services.helpers, ('render_template',))
        self.patch_all(services.host, ('service_running',))
        self.patch_all(services.puppet, ('_apply',))
        self.patch_all(services.templating.OSConfigRenderer, ('write_all',))

        self.test_config.config['openstack-origin'] = 'cloud:trusty-liberty'
        self.config.side_effect = self.test_config.get
        self.related_units.side_effect = self.relations.related_units
        self.relation_get.side_effect = self.relations.get
        self.relation_ids.side_effect = self.relations.relation_ids

        services.context.related_units = self.related_units
        services.context.relation_get = self.relation_get
        services.context.relation_ids = self.relation_ids
        services.context.config = self.config

        self.charm_dir.return_value = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.charm_dir.return_value)
        test.CharmTestCase.tearDown(self)

    def test_config_change(self):
        services.manage()

        self.assertTrue(self.render_template.called)
        self.assertTrue(self.service_restart.called)

    def _full_relation_data(self):
        return {
            'neutron_agents:4': {
                'nova-compute/0': {
                    'auth_host': self.nova[0],
                    'auth_port': self.nova[1],
                    'auth_protocol': self.nova[2],
                    'service_protocol': self.nova[3],
                    'service_port': self.nova[4],
                    'service_username': self.nova[5],
                    'service_password': self.nova[6],
                    'service_tenant_name': self.nova[7],
                    'keystone_host': self.nova[9],
                    'service_tenant': self.nova[10],
                    'region': self.nova[11],
                    'quantum_url': self.nova[12],
                    'quantum_host': self.nova[13],
                    'quantum_port': self.nova[14],
                    'private-address': self.nova[15],
                },
            },
            'midonet:8': {
                'midonet-api/0': {
                    'host': 'midonet_host',
                    'port': 8080,
                },
            },
            'amqp:2': {
                'rabbitmq/0': {
                    'clustered': False,
                    'private-address': '10.0.30.14',
                    'password': 'foopass',
                },
            },
        }
